use std::fmt::Display;
use std::io::Write;
use std::sync::Once;
use std::time::Instant;
use colored::Colorize;

pub struct Task(String, Instant, Once);

impl Task {
    pub fn new(msg: String, start: Instant) -> Self {
        Self(msg, start, Once::new())
    }

    pub fn start<Msg: Into<String>>(msg: Msg) -> Self {
        let msg = msg.into();
        print!("[ {} ] {}...\r", "RN".yellow(), msg);
        std::io::stdout().flush().unwrap();
        Self::new(msg, Instant::now())
    }

    fn end<T: Display>(&self, status: T) {
        self.2.call_once(||
            println!("[{}] {} - {:.2?}", status, self.0, self.1.elapsed())
        );
    }

    pub fn success(&self) {
        self.end(" OK ".green());
    }

    pub fn failure(&self) {
        self.end("FAIL".red());
    }
}

impl Drop for Task {
    fn drop(&mut self) {
        self.failure()
    }
}
